/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <string.h>
#include "DrvUSB.h"
#include "DrvSYS.h"
#include "Ccid_Sys.h"
#include "ccid_if.h"


#define USB_VID		0x0416  /* Vendor ID */
#define USB_PID		0xC146  /* Product ID */

//======================================================
#define INT_BUFFER_SIZE		64    /* Interrupt message buffer size */
#define BULK_BUFFER_SIZE	512   /* bulk message buffer size */

uint8_t UsbIntMessageBuffer[INT_BUFFER_SIZE];
uint8_t UsbMessageBuffer[BULK_BUFFER_SIZE];

//======================================================
// Discriptors
//======================================================
#define CCID_DEFAULT_INTERVAL_INT		0x08
#define CCID_DEFAULT_INTERVAL_BULK		0x00
#define CCID_IS_SELF_POWERED         	0
#define CCID_IS_REMOTE_WAKEUP        	0
#define CCID_MAX_POWER               	50  /* The unit is in 2mA. ex: 50 * 2mA = 100mA */

const uint8_t gau8DeviceDescriptor[LEN_DEVICE] =
{
	LEN_DEVICE,		    		/* bLength 				*/
	DESC_DEVICE,	    		/* bDescriptorType 		*/
	0x10, 0x01,		    		/* bcdUSB 				*/
	0x00,			    		/* bDeviceClass 		*/
	0x00,			    		/* bDeviceSubClass 		*/
	0x00,			    		/* bDeviceProtocol 		*/
	CCID_MAX_PACKET_SIZE_CTRL,	/* bMaxPacketSize0		*/
	USB_VID & 0x00FF,			/* Vendor ID 			*/
	(USB_VID & 0xFF00) >> 8, 
	USB_PID & 0x00FF,		/* Product ID 			*/
	(USB_PID & 0xFF00) >> 8, 
	0x00, 0x00,		    		/* bcdDevice 			*/
	0x01,			    		/* iManufacture 		*/
	0x02,			    		/* iProduct				*/
	0x00,			    		/* iSerialNumber 		*/
	0x01			    		/* bNumConfigurations 	*/
};

const uint8_t gau8ConfigDescriptor[] =
{
	// Configuration descriptor                                                                      
	LEN_CONFIG,		// bLength
	DESC_CONFIG,	// bDescriptorType
	// wTotalLength
	(LEN_CONFIG+LEN_INTERFACE+LEN_CCID+LEN_ENDPOINT*3) & 0x00FF,
	((LEN_CONFIG+LEN_INTERFACE+LEN_CCID+LEN_ENDPOINT*3) & 0xFF00) >> 8,
	0x01,			// bNumInterfaces
	0x01,			// bConfigurationValue
	0x00,			// iConfiguration
	0xC0,			// bmAttributes
	0x32,			// MaxPower

	// Interface descriptor (Interface 0 = Smart Card Reader)
 	LEN_INTERFACE,
 	DESC_INTERFACE,
 	0x00,           // bInterfaceNumber
 	0x00,           // bAlternateSetting
 	0x03,           // bNumEndpoints
 	0x0B,           // bInterfaceClass
 	0x00,           // bInterfaceSubClass
 	0x00,   		// bInterfaceProtocol
 	0x00,           // iInterface

	// CCID class descriptor
	0x36,		// bLength: CCID Descriptor size
	0x21,		// bDescriptorType: HID To be updated with CCID specific number
	0x10,		// bcdHID(LSB): CCID Class Spec release number (1.10)
	0x01,		// bcdHID(MSB)

	0x00,		// bMaxSlotIndex
	0x07,		// bVoltageSupport: 5v, 3v and 1.8v

	0x03,0x00,0x00,0x00,	// dwProtocols: supports T=0 and T=1
	0xA6,0x0E,0x00,0x00,	// dwDefaultClock: 3.75MHz /* 4 Mhz (0x00000FA0)*/
	0x4C,0x1D,0x00,0x00,	// dwMaximumClock: 7.5MHz  /* 4 Mhz (0x00000FA0)*/
	0x00,					// bNumClockSupported => no manual setting

	0x00,0x2A,0x00,0x00,	// dwDataRate: 10080 bps  /* 10752 bps (0x00002A00)*/
	0xB4,0xC4,0x04,0x00,	// dwMaxDataRate: 312500 bps  /* 129032 bps (0x0001F808)*/
	0x00,					// bNumDataRatesSupported => no manual setting

	0xFE,0x00,0x00,0x00,    /* dwMaxIFSD: 0 (T=0 only)   */
	0x00,0x00,0x00,0x00,    /* dwSynchProtocols  */
	0x00,0x00,0x00,0x00,    /* dwMechanical: no special characteristics */

	0x38,0x01,0x00,0x00,	// dwFeatures: clk, baud rate, voltage : automatic
							// CCID can set ICC in clock stop mode.
	0x0F,0x01,0x00,0x00,   	/* dwMaxCCIDMessageLength : Maximun block size + header*/
							/* 261 + 10   */

	0x00,			/* bClassGetResponse*/
	0x00,			/* bClassEnvelope */
	0x00,0x00,		/* wLcdLayout */
	0x00,			/* bPINSupport : no PIN verif and modif  */
	0x01,			/* bMaxCCIDBusySlots	*/

	// Endpoint 1 descriptor (Interrupt in)
 	LEN_ENDPOINT,                                   // bLength
 	DESC_ENDPOINT,                                  // bDescriptorType
 	CCID_INT_IN_EP_NUM | 0x80,                      // bEndpointAddress
 	EP_INT,                                         // bmAttributes
 	CCID_MAX_PACKET_SIZE_INT & 0x00FF,              // wMaxPacketSize
	(CCID_MAX_PACKET_SIZE_INT & 0xFF00) >> 8,         
 	CCID_DEFAULT_INTERVAL_INT,                 		// bInterval

	// Endpoint 2 descriptor (Bulk out)
 	LEN_ENDPOINT,                                   // bLength
 	DESC_ENDPOINT,                                  // bDescriptorType
 	CCID_BULK_OUT_EP_NUM,                           // bEndpointAddress
 	EP_BULK,                                        // bmAttributes
	CCID_MAX_PACKET_SIZE_BULK & 0x00FF,             // wMaxPacketSize
	(CCID_MAX_PACKET_SIZE_BULK & 0xFF00) >> 8,
 	CCID_DEFAULT_INTERVAL_BULK,                		// bInterval

	// Endpoint 2 descriptor (Bulk in)
 	LEN_ENDPOINT,                                   // bLength
 	DESC_ENDPOINT,                                  // bDescriptorType
 	CCID_BULK_IN_EP_NUM | 0x80,                     // bEndpointAddress
 	EP_BULK,                                        // bmAttributes
	CCID_MAX_PACKET_SIZE_BULK & 0x00FF,             // wMaxPacketSize
	(CCID_MAX_PACKET_SIZE_BULK & 0xFF00) >> 8,
 	CCID_DEFAULT_INTERVAL_BULK                 		// bInterval
};

const uint8_t gau8StringLang[] = 
{
	4,				/* bLength                  */
	DESC_STRING,	/* bDescriptorType          */
	0x09, 0x04      /* Language ID: USA(0x0409) */
};

const uint8_t gau8VendorStringDescriptor[] = 
{
	16,             /* bLength          */
	DESC_STRING,    /* bDescriptorType  */
	'N', 0, 'u', 0, 'v', 0, 'o', 0, 't', 0, 'o', 0, 'n', 0
};

const uint8_t gau8ProductStringDescriptor[] = 
{
	52,             /* bLength          */
	DESC_STRING,    /* bDescriptorType  */
	'N', 0, 'u', 0, 'M', 0, 'i', 0, 'c', 0, 'r', 0, 'o', 0, ' ', 0,
	'S', 0, 'm', 0, 'a', 0, 'r', 0, 't', 0, ' ', 0, 
	'C', 0, 'a', 0, 'r', 0, 'd', 0, ' ', 0, 
	'R', 0, 'e', 0, 'a', 0, 'd', 0, 'e', 0, 'r', 0
};

//======================================================
// global function
//======================================================
void RoughDelay(uint32_t t)
{
    volatile int32_t delay;

    delay = t;

    while(delay-- >= 0);
}

void my_memcpy(void *dest, void *src, int32_t size)
{
    int32_t i;

    for (i = 0; i < size; i++)
       *((uint8_t *)dest + i) = *((uint8_t *)src + i);
}

//======================================================
void UsbBulkInMessage(void)
{
	g_CCID_sDevice.UsbMessageLength = USB_MESSAGE_HEADER_SIZE + make32(&UsbMessageBuffer[OFFSET_DWLENGTH]);

	g_CCID_sDevice.pUsbMessageBuffer = UsbMessageBuffer;

	if (g_CCID_sDevice.isBulkInReady)
	{
		if(g_CCID_sDevice.UsbMessageLength >= CCID_MAX_PACKET_SIZE_BULK)
		{
			g_CCID_sDevice.isBulkInReady = 1;
	   		DrvUSB_DataIn(CCID_BULK_IN_EP_NUM, g_CCID_sDevice.pUsbMessageBuffer, CCID_MAX_PACKET_SIZE_BULK);

			g_CCID_sDevice.pUsbMessageBuffer += CCID_MAX_PACKET_SIZE_BULK;
			g_CCID_sDevice.UsbMessageLength -= CCID_MAX_PACKET_SIZE_BULK;
		}
		else
		{
	   		DrvUSB_DataIn(CCID_BULK_IN_EP_NUM, g_CCID_sDevice.pUsbMessageBuffer, g_CCID_sDevice.UsbMessageLength);
			g_CCID_sDevice.UsbMessageLength = 0;
			g_CCID_sDevice.isBulkInReady = 0;
		}
	}
}

void CCID_DispatchMessage(void)
{
	uint8_t ErrorCode;

	if(g_CCID_sDevice.isBulkOutReady)
	{
		switch(UsbMessageBuffer[OFFSET_BMESSAGETYPE])
		{
			case PC_TO_RDR_ICCPOWERON:
				ErrorCode = PC_to_RDR_IccPowerOn();
				RDR_to_PC_DataBlock(ErrorCode);
				break;
			case PC_TO_RDR_ICCPOWEROFF:
				ErrorCode = PC_to_RDR_IccPowerOff();
				RDR_to_PC_SlotStatus(ErrorCode);
				break;
			case PC_TO_RDR_GETSLOTSTATUS:
				ErrorCode = PC_to_RDR_GetSlotStatus();
				RDR_to_PC_SlotStatus(ErrorCode);
				break;
			case PC_TO_RDR_XFRBLOCK:
				ErrorCode = PC_to_RDR_XfrBlock();
				RDR_to_PC_DataBlock(ErrorCode);
				break;
			case PC_TO_RDR_GETPARAMETERS:
				ErrorCode = PC_to_RDR_GetParameters();
				RDR_to_PC_Parameters(ErrorCode);
				break;
			case PC_TO_RDR_RESETPARAMETERS:
				ErrorCode = PC_to_RDR_ResetParameters();
				RDR_to_PC_Parameters(ErrorCode);
				break;
			case PC_TO_RDR_SETPARAMETERS:
				ErrorCode = PC_to_RDR_SetParameters();
				RDR_to_PC_Parameters(ErrorCode);
				break;
			case PC_TO_RDR_ESCAPE:
				ErrorCode = PC_to_RDR_Escape();
				RDR_to_PC_Escape(ErrorCode);
				break;
			case PC_TO_RDR_ICCCLOCK:
				ErrorCode = PC_to_RDR_IccClock();
				RDR_to_PC_SlotStatus(ErrorCode);
				break;
			case PC_TO_RDR_ABORT:
				ErrorCode = PC_to_RDR_Abort();
				RDR_to_PC_SlotStatus(ErrorCode);
				break;
			case PC_TO_RDR_SETDATARATEANDCLOCKFREQUENCY:
			case PC_TO_RDR_SECURE:
			case PC_TO_RDR_T0APDU:
			case PC_TO_RDR_MECHANICAL:
			default:
				CmdNotSupported();
				break;
		}

		UsbBulkInMessage();
		g_CCID_sDevice.isBulkOutReady = 0;
	}
}

/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* FUNCTION                                                                                                */
/*      BulkOutData()	          	                                                                       */
/*                                                                                                         */
/* DESCRIPTION                                                                                             */
/*      To get data from Bulk OUT to buffer.                                                               */
/*                                                                                                         */
/* INPUTS                                                                                                  */
/*      pi16src    The data buffer of ISO OUT.                                                             */
/*      i16Samples The sample number in data buffer.                                                       */
/*                                                                                                         */
/* OUTPUTS                                                                                                 */
/*      None                            				                                                   */
/*                                                                                                         */
/* RETURN                                                                                                  */
/*                                                                                                         */
/*      None                                                                                               */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
void BulkOutData(uint8_t *psrc, uint32_t len)
{
	my_memcpy(&UsbMessageBuffer[0], psrc, len);

	if(len >= 0x0A && len != 0xFF)
	{
		/* Calculate number of byte to receive to finish the message  */
		g_CCID_sDevice.UsbMessageLength = USB_MESSAGE_HEADER_SIZE + make32(&UsbMessageBuffer[OFFSET_DWLENGTH]);

		g_CCID_sDevice.UsbMessageLength -= (int) len;
		/* Prepare next reception if whole message not received */
		if(g_CCID_sDevice.UsbMessageLength > 0)
			g_CCID_sDevice.pUsbMessageBuffer = UsbMessageBuffer + len;

		if(g_CCID_sDevice.UsbMessageLength == 0)
			g_CCID_sDevice.isBulkOutReady = 1;
		if(g_CCID_sDevice.UsbMessageLength < 0)
		{
			UsbMessageBuffer[OFFSET_DWLENGTH] = 0xFF;
			UsbMessageBuffer[OFFSET_DWLENGTH+1] = 0xFF;
			UsbMessageBuffer[OFFSET_DWLENGTH+2] = 0xFF;
			UsbMessageBuffer[OFFSET_DWLENGTH+3] = 0xFF;
			g_CCID_sDevice.isBulkOutReady = 1;
		}
	}
	CCID_DispatchMessage();
}

void CCID_Init(void *pfnBulkOutData)
{
	CCID_Open(pfnBulkOutData);
	CCID_SetBuffer(UsbMessageBuffer, UsbIntMessageBuffer);
}

void SC_CardState(void)
{
	RDR_to_PC_NotifySlotChange();
	g_CCID_sDevice.u32IntInSize = 2;
	DrvUSB_DataIn(CCID_INT_IN_EP_NUM, g_CCID_sDevice.pu8IntInBuf, g_CCID_sDevice.u32IntInSize);
}

/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* FUNCTION                                                                                                */
/*      CCID_MainProcess()	      	                                                                       */
/*                                                                                                         */
/* DESCRIPTION                                                                                             */
/*      The main loop of UAC funciton.                                                                     */
/*                                                                                                         */
/* INPUTS                                                                                                  */
/*      None                                                                                               */
/*                                                                                                         */
/* OUTPUTS                                                                                                 */
/*      None                            				                                                   */
/*                                                                                                         */
/* RETURN                                                                                                  */
/*                                                                                                         */
/*      None                                                                                               */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
int32_t CCID_MainProcess(void)
{
	E_DRVUSB_STATE eUsbState;

    /* Install the USB event handler to USB event callback */
	DrvUSB_Open((void *)0);
		
   	CCID_Init(BulkOutData);
	while(1)
	{
		/* Disable USB-related interrupts. */
		_DRVUSB_ENABLE_MISC_INT(0);

		/* Enable float-detection interrupt. */
		_DRVUSB_ENABLE_FLDET_INT();

		// Wait for USB connected.
		while (1)
		{
			// Order here is significant.
			// Give a chance to handle remaining events before exiting this loop.
			eUsbState = DrvUSB_GetUsbState();

			DrvUSB_DispatchEvent();

			if ( eUsbState >= eDRVUSB_ATTACHED &&
			     eUsbState != eDRVUSB_SUSPENDED )
			{
				break;
			}
		}

		/* init smart card */
               
		// Enable USB-related interrupts.
		_DRVUSB_ENABLE_MISC_INT(INTEN_WAKEUP | INTEN_WAKEUPEN | INTEN_FLDET | INTEN_USB | INTEN_BUS);
 
		/* Poll and handle USB events. */
		while(1)
		{
			DrvUSB_DispatchEvent();
			eUsbState = DrvUSB_GetUsbState();

			if (eUsbState == eDRVUSB_DETACHED)
			{
				break;
			}
		}

		/* exit smart card */


		// Disable USB-related interrupts.
		_DRVUSB_ENABLE_MISC_INT(0);		
	}
}




