/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <string.h>
#include "DrvUSB.h"
#include "Ccid_Sys.h"
#include "ccid_if.h"


//#define DBG_PRINTF      printf
#define DBG_PRINTF(...)


#define Maximum(a,b)		((a)>(b) ? (a) : (b))
#define Minimum(a,b)		((a)<(b) ? (a) : (b))

S_CCID_DEVICE g_CCID_sDevice;

/* The CCID control and data callback */
CCID_BULK_OUT pfnCcidBulkOutMessage;

void CCID_BulkInCallback(void * pVoid);
void CCID_BulkOutCallback(void * pVoid);
void CCID_IntInCallback(void * pVoid);
void CCID_CtrlGetDescriptorIn(void * pVoid);
void CCID_CtrlSetupGetDescriptor(void * pVoid);
void CCID_CtrlGetDescriptorOut(void * pVoid);
void CCID_ClassCtrlAbortRequest(void * pVoid);
void CCID_ClassCtrlRequest(void * pVoid);
void CCID_ClassCtrlRequestIn(void * pVoid);
void CCID_UsbStartCallBack(void * pVoid);

//=============================================================
/* Endpoint description */
S_DRVUSB_EP_CTRL sEpDescription[] = 
{
	{CTRL_EP_NUM          | EP_INPUT,  CCID_MAX_PACKET_SIZE_CTRL, NULL}, /* EP Id 0, EP Addr 0, input , max packe size = 16 */
	{CTRL_EP_NUM          | EP_OUTPUT, CCID_MAX_PACKET_SIZE_CTRL, NULL}, /* EP Id 1, EP Addr 0, output , max packe size = 16 */
	{CCID_BULK_IN_EP_NUM  | EP_INPUT,  CCID_MAX_PACKET_SIZE_BULK, NULL}, /* EP Id 2, EP Addr 2, input , max packe size = 64 */
	{CCID_BULK_OUT_EP_NUM | EP_OUTPUT, CCID_MAX_PACKET_SIZE_BULK, NULL}, /* EP Id 3, EP Addr 2, output , max packe size = 64 */
	{CCID_INT_IN_EP_NUM   | EP_INPUT,  CCID_MAX_PACKET_SIZE_INT,  NULL}, /* EP Id 4, EP Addr 3, input , max packe size = 16 */
	{0,0,0}
};

//bus event call back function
S_DRVUSB_EVENT_PROCESS g_sBusOps[6] = 
{
	{NULL, NULL},//attach
	{NULL, NULL},//detach
	{DrvUSB_BusResetCallback, &g_CCID_sDevice}, /* bus reset event callback */
	{NULL, NULL},                               /* bus suspend event callback */
	{NULL, NULL},                               /* bus resume event callback */
	{DrvUSB_CtrlSetupAck, &g_CCID_sDevice},     /* setup event callback */
};

//USB event call back function
S_DRVUSB_EVENT_PROCESS g_sUsbOps[12] = 
{
	{DrvUSB_CtrlDataInAck, &g_CCID_sDevice},  /* ctrl pipe0 (EP address 0) In ACK callback */
	{DrvUSB_CtrlDataOutAck, &g_CCID_sDevice}, /* ctrl pipe0 (EP address 0) Out ACK callback */
	{NULL, NULL},							  /* EP address 1 In ACK callback */
	{NULL, NULL},                             /* EP address 1 Out ACK callback */
	{CCID_BulkInCallback, &g_CCID_sDevice},   /* EP address 2 In ACK callback */  
	{CCID_BulkOutCallback, &g_CCID_sDevice},  /* EP address 2 Out ACK callback */  
	{CCID_IntInCallback, &g_CCID_sDevice},    /* EP address 3 In ACK callback */  
	{NULL, NULL},                             /* EP address 3 Out ACK callback */  
	{NULL, NULL},                             /* EP address 4 In ACK callback */  
	{NULL, NULL},                             /* EP address 4 Out ACK callback */  
	{NULL, NULL},                             /* EP address 5 In ACK callback */  
	{NULL, NULL},                             /* EP address 5 Out ACK callback */  
};


/* ctrl pipe call back.                                                                   */
/* it will be call by DrvUSB_CtrlSetupAck, DrvUSB_CtrlDataInAck and DrvUSB_CtrlDataOutAck */
/* if in ack handler and out ack handler is 0, default handler will be called             */
S_DRVUSB_CTRL_CALLBACK_ENTRY g_asCtrlCallbackEntry[] =
{  /* request type,command     ,setup ack handler         , in ack handler      ,out ack handler,  parameter */
	{REQ_STANDARD, SET_ADDRESS, DrvUSB_CtrlSetupSetAddress, DrvUSB_CtrlDataInSetAddress, 0, &g_CCID_sDevice},
	{REQ_STANDARD, CLEAR_FEATURE, DrvUSB_CtrlSetupClearSetFeature, 0, 0, &g_CCID_sDevice},
	{REQ_STANDARD, SET_FEATURE, DrvUSB_CtrlSetupClearSetFeature, 0, 0, &g_CCID_sDevice},
	{REQ_STANDARD, GET_CONFIGURATION, DrvUSB_CtrlSetupGetConfiguration, 0, 0, &g_CCID_sDevice},
	{REQ_STANDARD, GET_STATUS, DrvUSB_CtrlSetupGetStatus, 0, 0, &g_CCID_sDevice},
	{REQ_STANDARD, GET_INTERFACE, DrvUSB_CtrlSetupGetInterface, 0, 0, &g_CCID_sDevice},
	{REQ_STANDARD, SET_INTERFACE, DrvUSB_CtrlSetupSetInterface, 0, 0, &g_CCID_sDevice},
	{REQ_STANDARD, GET_DESCRIPTOR, CCID_CtrlSetupGetDescriptor, CCID_CtrlGetDescriptorIn, CCID_CtrlGetDescriptorOut, &g_CCID_sDevice},
	{REQ_STANDARD, SET_CONFIGURATION, DrvUSB_CtrlSetupSetConfiguration, 0, 0, &g_CCID_sDevice},
	{REQ_CLASS, CCID_ABORT, CCID_ClassCtrlAbortRequest, 0, 0, &g_CCID_sDevice},
	{REQ_CLASS, CCID_GET_CLOCK_FREQUENCIES, CCID_ClassCtrlRequest, CCID_ClassCtrlRequestIn, 0, &g_CCID_sDevice},
	{REQ_CLASS, CCID_GET_DATA_RATES, CCID_ClassCtrlRequest, CCID_ClassCtrlRequestIn, 0, &g_CCID_sDevice},
};

/* The structure registered to USB driver */
S_DRVUSB_CLASS sCcidUsbClass = 
{
	(void *)&g_CCID_sDevice, 
	CCID_UsbStartCallBack,
	CCID_IsConfigureValue
};


static const uint8_t * gpu8UsbBuf = 0;
static uint32_t gu32BytesInUsbBuf = 0;
static int8_t gIsOverRequest = 0;

/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* FUNCTION                                                                                                */
/*      CCID_Open()	                       	                                                               */
/*                                                                                                         */
/* DESCRIPTION                                                                                             */
/*      To initial all descriptors and control handlers of USB Audio Class.                                */
/*                                                                                                         */
/* INPUTS                                                                                                  */
/*      pVoid   The function point of UAC device                                                           */
/*                                                                                                         */
/* OUTPUTS                                                                                                 */
/*      None                            				                                                   */
/*                                                                                                         */
/* RETURN                                                                                                  */
/*      E_SUCCESS                                                                                          */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
int32_t CCID_Open(void *pfnBulkOutData)
{		
	g_CCID_sDevice.device = (void *)DrvUSB_InstallClassDevice(&sCcidUsbClass);
    pfnCcidBulkOutMessage = (CCID_BULK_OUT)pfnBulkOutData;

	DrvUSB_InstallCtrlHandler(g_CCID_sDevice.device, g_asCtrlCallbackEntry, 
					sizeof(g_asCtrlCallbackEntry) / sizeof(g_asCtrlCallbackEntry[0]));

	return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* FUNCTION                                                                                                */
/*      CCID_Close()	                       	                                                               */
/*                                                                                                         */
/* DESCRIPTION                                                                                             */
/*      None.                                                                                              */
/*                                                                                                         */
/* INPUTS                                                                                                  */
/*      None                                                                                               */
/*                                                                                                         */
/* OUTPUTS                                                                                                 */
/*      None                            				                                                   */
/*                                                                                                         */
/* RETURN                                                                                                  */
/*      E_SUCCESS                                                                                          */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
void CCID_Close(void)
{

}

/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* FUNCTION                                                                                                */
/*      CCID_Reset()	                       	                                                               */
/*                                                                                                         */
/* DESCRIPTION                                                                                             */
/*      The function to reset endpoint settings. It would be called at set configuration or set interface. */
/*                                                                                                         */
/* INPUTS                                                                                                  */
/*      psDevice   The function point of UAC device                                                        */
/*                                                                                                         */
/* OUTPUTS                                                                                                 */
/*      None                            				                                                   */
/*                                                                                                         */
/* RETURN                                                                                                  */
/*      E_SUCCESS                                                                                          */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
void CCID_Reset(S_CCID_DEVICE *psDevice)
{
	DrvUSB_Reset(CCID_INT_IN_EP_NUM);
	DrvUSB_Reset(CCID_BULK_IN_EP_NUM);
	DrvUSB_Reset(CCID_BULK_OUT_EP_NUM);
}

void CCID_Start(S_CCID_DEVICE *psDevice)
{
    /* To prepare the first IN report if it is necessary */
	g_CCID_sDevice.isDeviceReady = 1;
	RDR_to_PC_NotifySlotChange();
	g_CCID_sDevice.u32IntInSize = 2;
	DrvUSB_DataIn(CCID_INT_IN_EP_NUM, g_CCID_sDevice.pu8IntInBuf, g_CCID_sDevice.u32IntInSize);

    /* To prepare the first OUT report if it is necessary */
	DrvUSB_DataOutTrigger(CCID_BULK_OUT_EP_NUM, CCID_MAX_PACKET_SIZE_BULK);
}


void CCID_UsbStartCallBack(void * pVoid)
{
	CCID_Reset((S_CCID_DEVICE *)pVoid);
	CCID_Start((S_CCID_DEVICE *)pVoid);
}



/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* FUNCTION                                                                                                */
/*      CCID_IsConfigureValue()	                  	                                                               */
/*                                                                                                         */
/* DESCRIPTION                                                                                             */
/*      To check if it is the specified bConfigurationValue in the confiuration descriptor.                */
/*                                                                                                         */
/* INPUTS                                                                                                  */
/*      u8ConfigureValue   The bConfigurationValue to check.                                               */
/*                                                                                                         */
/* OUTPUTS                                                                                                 */
/*      None                            				                                                   */
/*                                                                                                         */
/* RETURN                                                                                                  */
/*      E_SUCCESS                                                                                          */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
int32_t CCID_IsConfigureValue(uint8_t u8ConfiguationValue)
{
	return (u8ConfiguationValue == gau8ConfigDescriptor[5]);
}


void CCID_PrepareDescriptors(const uint8_t *pu8Descriptor, uint32_t u32DescriptorSize, uint32_t u32RequestSize, uint32_t u32MaxPacketSize)
{
    gu32BytesInUsbBuf = u32RequestSize;
    if(u32RequestSize > u32DescriptorSize)
    {
        gu32BytesInUsbBuf = u32DescriptorSize;
        gIsOverRequest = 1;
    }
    gpu8UsbBuf = pu8Descriptor;

    DBG_PRINTF("Get descriptor 0x%08x %d size.\n", pu8Descriptor, u32DescriptorSize);

	if(gu32BytesInUsbBuf < u32MaxPacketSize)
	{
	    DrvUSB_DataIn(0, gpu8UsbBuf, gu32BytesInUsbBuf); 
	    gpu8UsbBuf = 0;
	    gu32BytesInUsbBuf = 0;   
	}
	else
	{
		DrvUSB_DataIn(0, gpu8UsbBuf, u32MaxPacketSize);
		gpu8UsbBuf += u32MaxPacketSize;
		gu32BytesInUsbBuf -= u32MaxPacketSize;
    }
}


void CCID_CtrlGetDescriptorOut(void * pVoid)
{
    gu32BytesInUsbBuf = 0;
    gpu8UsbBuf = 0;
    gIsOverRequest = 0;
}

void CCID_CtrlGetDescriptorIn(void * pVoid)
{
    uint32_t u32Len;


    DBG_PRINTF(" >>> 0x%08x %d size.\n", gpu8UsbBuf, gu32BytesInUsbBuf);
	
    if(gpu8UsbBuf)
    {
        if(gu32BytesInUsbBuf == 0)
        {
            /* Zero packet */
    		DrvUSB_DataIn(0, gpu8UsbBuf, 0);
    		gpu8UsbBuf = 0;
            _DRVUSB_TRIG_EP(1,0x00);
        }
        else
        {
            u32Len = Minimum(gu32BytesInUsbBuf, CCID_MAX_PACKET_SIZE_CTRL);
    		DrvUSB_DataIn(0, gpu8UsbBuf, u32Len);
    		gpu8UsbBuf += u32Len;
    		gu32BytesInUsbBuf -= u32Len;
    		
    		if(gu32BytesInUsbBuf == 0)
    		{
                if(u32Len < CCID_MAX_PACKET_SIZE_CTRL)
                {
                    /* This should be last IN packet due to it is less than CCID_MAX_PACKET_SIZE_CTRL */
                    gpu8UsbBuf = 0;
                }
                else
                {
                    if(!gIsOverRequest)
                    {
    		            /* This should be the last IN packet because there is no more data to 
                           transfer and it is not over request transfer */
                        gpu8UsbBuf = 0;
                    }
                }
                _DRVUSB_TRIG_EP(1,0x00);
            }
        }
    }
    else
    {
  	    /* The EP id 1 should always be used as control (OUT) endpoint */
		_DRVUSB_TRIG_EP(1,0x00);
    }
}


/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* FUNCTION                                                                                                */
/*      UAC_CtrlSetupGetDescriptor()	       	                                                           */
/*                                                                                                         */
/* DESCRIPTION                                                                                             */
/*      The callback funciton for get descriptor request ack event.                                        */
/*                                                                                                         */
/* INPUTS                                                                                                  */
/*      pVoid   A pointer of UAC device.                                                                   */
/*                                                                                                         */
/* OUTPUTS                                                                                                 */
/*      None                            				                                                   */
/*                                                                                                         */
/* RETURN                                                                                                  */
/*      None                                                                                               */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
void CCID_CtrlSetupGetDescriptor(void * pVoid)
{
	S_CCID_DEVICE *psDevice = (S_CCID_DEVICE *) pVoid;
	S_DRVUSB_DEVICE *pInfraDevice = (S_DRVUSB_DEVICE *)psDevice->device;
	uint16_t u16Len;

	u16Len = 0;
	u16Len = pInfraDevice->au8Setup[7];
	u16Len <<= 8;
	u16Len += pInfraDevice->au8Setup[6];

    gIsOverRequest = 0;	
	gu32BytesInUsbBuf = 0;
	gpu8UsbBuf = 0;
	switch (pInfraDevice->au8Setup[3])
	{
		/* Get Device Descriptor */
		case DESC_DEVICE:
		{
            CCID_PrepareDescriptors(gau8DeviceDescriptor, LEN_DEVICE, u16Len, CCID_MAX_PACKET_SIZE_CTRL);
    
    	    /* Prepare the OUT to avoid HOST stop data phase without all data transfered. */
    		//_DRVUSB_TRIG_EP(1,0x00);

		    break;
		}
		/* Get Configuration Descriptor */
		case DESC_CONFIG:
		{
            CCID_PrepareDescriptors(gau8ConfigDescriptor, gau8ConfigDescriptor[2] + gau8ConfigDescriptor[3]*256, u16Len, CCID_MAX_PACKET_SIZE_CTRL);
			break;
		}
		/* Get String Descriptor */
		case DESC_STRING:
		{
    		// Get Language
    		if (pInfraDevice->au8Setup[2] == 0)
    		{
                CCID_PrepareDescriptors(gau8StringLang, 4, u16Len, CCID_MAX_PACKET_SIZE_CTRL);
    		}
    		else
    		{
    			// Get String Descriptor
    			switch (pInfraDevice->au8Setup[2])
    			{
    			case 1:
                    CCID_PrepareDescriptors((const uint8_t *)gau8VendorStringDescriptor, gau8VendorStringDescriptor[0], u16Len, CCID_MAX_PACKET_SIZE_CTRL);
    				break;

    			case 2:
                    CCID_PrepareDescriptors((const uint8_t *)gau8ProductStringDescriptor, gau8ProductStringDescriptor[0], u16Len, CCID_MAX_PACKET_SIZE_CTRL);
      				break;

    			default:
    				/* Not support. Reply STALL. */
    				DrvUSB_ClrCtrlReadyAndTrigStall();
    			}
    		}
    		break;
		}

		default:
			/* Not support. Reply STALL. */
			DrvUSB_ClrCtrlReadyAndTrigStall();
			break;
	}
}

//=====================================================
void CCID_IntInCallback(void * pVoid)
{
//printf("int-in\n");

	if (g_CCID_sDevice.isDeviceReady)
	{
		RDR_to_PC_NotifySlotChange();
		g_CCID_sDevice.u32IntInSize = 2;
		DrvUSB_DataIn(CCID_INT_IN_EP_NUM, g_CCID_sDevice.pu8IntInBuf, g_CCID_sDevice.u32IntInSize);
		g_CCID_sDevice.isDeviceReady = 0;
	}
	if (!g_CCID_sDevice.isBulkOutReady)
		DrvUSB_DataOutTrigger(CCID_BULK_OUT_EP_NUM, CCID_MAX_PACKET_SIZE_BULK);
}


void CCID_BulkInCallback(void * pVoid)
{
    S_CCID_DEVICE* psDevice = (S_CCID_DEVICE*) pVoid;

	if (psDevice->isBulkInReady)
	{
		if(psDevice->UsbMessageLength >= CCID_MAX_PACKET_SIZE_BULK)
		{
			psDevice->isBulkInReady = 1;
   			DrvUSB_DataIn(CCID_BULK_IN_EP_NUM, psDevice->pUsbMessageBuffer, CCID_MAX_PACKET_SIZE_BULK);

			psDevice->pUsbMessageBuffer += CCID_MAX_PACKET_SIZE_BULK;
			psDevice->UsbMessageLength -= CCID_MAX_PACKET_SIZE_BULK;
		}
		else
		{
	   		DrvUSB_DataIn(CCID_BULK_IN_EP_NUM, psDevice->pUsbMessageBuffer, psDevice->UsbMessageLength);
			psDevice->UsbMessageLength = 0;
			psDevice->isBulkInReady = 0;
		}
	}
	if (!psDevice->isBulkOutReady)
		DrvUSB_DataOutTrigger(CCID_BULK_OUT_EP_NUM, CCID_MAX_PACKET_SIZE_BULK);
}

void CCID_BulkOutCallback(void * pVoid)
{
    uint32_t size;
    uint8_t *pBuf;

    pBuf = DrvUSB_GetOutData(CCID_BULK_OUT_EP_NUM, &size);
    pfnCcidBulkOutMessage(pBuf, size);

	/* trigger next out packet */
	if (g_CCID_sDevice.UsbMessageLength > 0)
		DrvUSB_DataOutTrigger(CCID_BULK_OUT_EP_NUM, CCID_MAX_PACKET_SIZE_BULK);
}

//================================================================
void CCID_ClassCtrlAbortRequest(void * pVoid)
{
	S_DRVUSB_DEVICE *psDevice = (S_DRVUSB_DEVICE *)((S_CCID_DEVICE *)pVoid)->device;

	if(psDevice->au8Setup[1] == CCID_ABORT)
	{
		g_CCID_sDevice.bAbortRequestFlag = 1;
		_DRVUSB_SET_EP_TOG_BIT(0, FALSE);
		_DRVUSB_TRIG_EP(0, 0);
	}
	else
	{
		// Not support. Reply STALL.
        //DBG_PRINTF(" - Unknown\n");
		_DRVUSB_CLEAR_EP_READY_AND_TRIG_STALL(0);
		_DRVUSB_CLEAR_EP_READY_AND_TRIG_STALL(1);
	}
}

void CCID_ClassCtrlRequest(void * pVoid)
{
	S_DRVUSB_DEVICE *psDevice = (S_DRVUSB_DEVICE *)((S_CCID_DEVICE *)pVoid)->device;
    uint8_t pData[1] = {0};

	if(psDevice->au8Setup[1] == CCID_GET_CLOCK_FREQUENCIES)
	{
		DrvUSB_DataIn(CTRL_EP_NUM, (uint8_t *)pData, sizeof(pData));
	}
	else if(psDevice->au8Setup[1] == CCID_GET_DATA_RATES)
	{
		DrvUSB_DataIn(CTRL_EP_NUM, (uint8_t *)pData, sizeof(pData));
	}
	else
	{
		// Not support. Reply STALL.
        //DBG_PRINTF(" - Unknown\n");
		_DRVUSB_CLEAR_EP_READY_AND_TRIG_STALL(0);
		_DRVUSB_CLEAR_EP_READY_AND_TRIG_STALL(1);
	}
}

void CCID_ClassCtrlRequestIn(void * pVoid)
{
	_DRVUSB_TRIG_EP(1,0x00);
}


//===========================================================
int32_t CCID_SetBuffer(uint8_t* pu8Bulk, uint8_t* pu8Int)
{
	if (pu8Bulk == NULL)
	{
		return -1;
	}
	if (pu8Int == NULL)
	{
		return -1;
	}

	g_CCID_sDevice.pu8BulkBuf = pu8Bulk;
	g_CCID_sDevice.pu8IntInBuf = pu8Int;

	return E_SUCCESS;
}

