/****************************************************************
 *                                                              *
 * Copyright (c) Nuvoton Technology Corp. All rights reserved.  *
 *                                                              *
 ****************************************************************/
#ifndef __CCID_SYS_H__
#define __CCID_SYS_H__


#ifdef  __cplusplus
extern "C"
{
#endif

/* For CCID */
#define LEN_CCID			0x36

//======================================================
#define CTRL_EP_NUM				0
#define CCID_INT_IN_EP_NUM		3
#define CCID_BULK_OUT_EP_NUM	2
#define CCID_BULK_IN_EP_NUM		2

// Max Packet Size
#define	CCID_MAX_PACKET_SIZE_CTRL		16
#define CCID_MAX_PACKET_SIZE_INT		16
#define CCID_MAX_PACKET_SIZE_BULK		64


//======================================================
/* CCID Class REQUEST */
#define 	CCID_ABORT          					0x01
#define 	CCID_GET_CLOCK_FREQUENCIES				0x02
#define 	CCID_GET_DATA_RATES						0x03

/* CCID Bulk-Out Message */
#define		PC_TO_RDR_ICCPOWERON					0x62
#define		PC_TO_RDR_ICCPOWEROFF					0x63
#define		PC_TO_RDR_GETSLOTSTATUS					0x65
#define		PC_TO_RDR_XFRBLOCK						0x6F
#define		PC_TO_RDR_GETPARAMETERS					0x6C
#define		PC_TO_RDR_RESETPARAMETERS				0x6D
#define		PC_TO_RDR_SETPARAMETERS					0x61
#define		PC_TO_RDR_ESCAPE						0x6B
#define		PC_TO_RDR_ICCCLOCK						0x6E
#define		PC_TO_RDR_T0APDU						0x6A
#define		PC_TO_RDR_SECURE						0x69
#define		PC_TO_RDR_MECHANICAL					0x71
#define		PC_TO_RDR_ABORT							0x72
#define		PC_TO_RDR_SETDATARATEANDCLOCKFREQUENCY	0x73

/* CCID Bulk-In Message */
#define		RDR_TO_PC_DATABLOCK						0x80
#define		RDR_TO_PC_SLOTSTATUS					0x81
#define		RDR_TO_PC_PARAMETERS					0x82
#define		RDR_TO_PC_ESCAPE						0x83
#define		RDR_TO_PC_DATARATEANDCLOCKFREQUENCY		0x84

/* CCID Interrupt-In Message */
#define		RDR_TO_PC_NOTIFYSLOTCHANGE				0x50
#define		RDR_TO_PC_HARDWAREERROR					0x51


//========================================================
/* 		Variables and Definition of authorized values	*/
//========================================================
#define USB_MESSAGE_BUFFER_MAX_LENGTH		0x010F
#define ICC_MESSAGE_BUFFER_MAX_LENGTH		0x0105
#define USB_MESSAGE_HEADER_SIZE				10
/* Offsets in UsbMessageBuffer for Bulk Out messages */
	#define OFFSET_BMESSAGETYPE				0x00
	#define OFFSET_DWLENGTH					0x01
	#define OFFSET_BSLOT					0x05
	#define OFFSET_BSEQ						0x06
	#define OFFSET_BPOWERSELECT				0x07
	#define OFFSET_BBWI						0x07
	#define OFFSET_BPROTOCOLNUM_OUT			0x07
	#define OFFSET_BCLOCKCOMMAND			0x07
	#define OFFSET_BMCHANGES				0x07
	#define OFFSET_BFUNCTION				0x07
	#define OFFSET_ABRFU_3B					0x07
	#define OFFSET_WLEVELPARAMETER			0x08
	#define OFFSET_BCLASSGETRESPONSE		0x08
	#define OFFSET_ABRFU_2B					0x08
	#define OFFSET_BCLASSENVELOPE			0x09
	#define OFFSET_ABDATA					0x0A
/* Offsets in UsbMessageBuffer for Bulk In messages */
	#define OFFSET_BSTATUS					0x07
	#define OFFSET_BERROR					0x08
	#define OFFSET_BCHAINPARAMETER			0x09
	#define OFFSET_BCLOCKSTATUS				0x09
	#define OFFSET_BPROTOCOLNUM_IN			0x09
	#define OFFSET_ABPROTOCOLDATASTRUCTURE	0x0A
	#define OFFSET_BRFU						0x09

/* Offsets in UsbMessageBuffer for Int In messages */
	#define OFFSET_INT_BMESSAGETYPE			0x00
	#define OFFSET_INT_BMSLOTICCSTATE		0x01
	#define OFFSET_INT_BSLOT				0x01
	#define OFFSET_INT_BSEQ					0x02
	#define OFFSET_INT_BHARDWAREERRORCODE	0x03

//-------------------------------------------------------------------------------------------------
__inline uint32_t make32(uint8_t *buf)
{
	return ((uint32_t) buf[3] << 24) | ((uint32_t) buf[2] << 16) |
			((uint32_t) buf[1] << 8) | ((uint32_t) buf[0]);
}

__inline uint16_t make16(uint8_t *buf)
{
	return ((uint16_t) buf[1] << 8) | ((uint16_t) buf[0]);
}


    
typedef struct
{	
	uint8_t isDeviceReady;
	uint8_t bAbortRequestFlag;
    uint8_t isBulkOutReady;
    uint8_t isBulkInReady;

	uint8_t *pu8BulkBuf;
	uint8_t *pu8IntInBuf;
	uint8_t *pUsbMessageBuffer;
	uint32_t u32BulkSize;
	uint32_t u32IntInSize;

	int32_t UsbMessageLength;

    void * *device;

} S_CCID_DEVICE;

extern S_CCID_DEVICE g_CCID_sDevice;

typedef void (*CCID_BULK_OUT)(uint8_t *psrc, uint32_t len);

//======================================================
extern const uint8_t gau8DeviceDescriptor[];
extern const uint8_t gau8ConfigDescriptor[];
extern const uint8_t gau8VendorStringDescriptor[];
extern const uint8_t gau8ProductStringDescriptor[];

extern const uint8_t gau8StringLang[];

//=======================================================
uint32_t CCID_GetVersion(void);
int32_t CCID_Open(void * pVoid);
void CCID_Close(void);
void CCID_Reset(S_CCID_DEVICE *psDevice);
int32_t CCID_IsConfigureValue(uint8_t u8ConfigureValue);
void CCID_SendOneAudioPacket(uint8_t *pbyAudioData, uint8_t u8AudioSize);
void CCID_Init(void *pfnBulkOutData);
int32_t CCID_SetBuffer(uint8_t* pu8Bulk, uint8_t* pu8Int);

#ifdef  __cplusplus
}
#endif

#endif // __CCID_SYS_H__
