/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <string.h>
#include "Driver\DrvUSB.h"
#include "ccid_sys.h"
#include "ccid_if.h"

/*===== for test =====*/
uint8_t PowerOnBuf[] = {0x3B, 0x7D, 0x94, 0x00, 0x00, 0x57, 0x44, 0x29,
						0x49, 0xF8, 0x86, 0x93, 0x00, 0x43, 0x13, 0x16,
						0x3A, 0x0B};
/*===== end for test =====*/

extern uint8_t UsbIntMessageBuffer[];
extern uint8_t UsbMessageBuffer[];

//======================================================
// Bulk-Out Message Functions
//======================================================
uint8_t PC_to_RDR_IccPowerOn(void)
{
	uint8_t ErrorCode, Voltage;
	uint32_t AtrSize;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if (make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_BPOWERSELECT] >= 0x04)
		return SLOTERR_BAD_POWERSELECT;
	if (make16(&UsbMessageBuffer[OFFSET_ABRFU_2B]) != 0)
		return SLOTERR_BAD_ABRFU_2B;

	if( g_CCID_sDevice.bAbortRequestFlag )
		return SLOTERR_CMD_ABORTED;

	Voltage = UsbMessageBuffer[OFFSET_BPOWERSELECT];
//	ErrorCode = IccPowerOn(Voltage, &UsbMessageBuffer[OFFSET_ABDATA], &AtrSize);

	/*===== for test =====*/
	if (UsbMessageBuffer[OFFSET_BPOWERSELECT] == 0x3)
		ErrorCode = SLOTERR_ICC_MUTE;
	else if (UsbMessageBuffer[OFFSET_BPOWERSELECT] == 0x2)
	{
		AtrSize = 0x12;
		ErrorCode = SLOT_NO_ERROR;
		UsbMessageBuffer[OFFSET_BSTATUS] = 0x40;
		memcpy(&UsbMessageBuffer[OFFSET_ABDATA], &PowerOnBuf[0], 0x12);
	}
	/*===== end for test =====*/

	if( g_CCID_sDevice.bAbortRequestFlag )
		return SLOTERR_CMD_ABORTED;

	if(ErrorCode == SLOT_NO_ERROR)
	{
		UsbMessageBuffer[OFFSET_DWLENGTH] = (uint8_t) AtrSize;
		UsbMessageBuffer[OFFSET_DWLENGTH+1] = (uint8_t) (AtrSize>>8);
		UsbMessageBuffer[OFFSET_DWLENGTH+2] = (uint8_t) (AtrSize>>16);
		UsbMessageBuffer[OFFSET_DWLENGTH+3] = (uint8_t) (AtrSize>>24);
		return SLOT_NO_ERROR;
	}

//	VccOff();

	return ErrorCode;
}

uint8_t PC_to_RDR_IccPowerOff(void)
{
	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if (make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_ABRFU_3B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_3B+1] != 0
		||UsbMessageBuffer[OFFSET_ABRFU_3B+2] != 0)
		return SLOTERR_BAD_ABRFU_3B;

//	VccOff();
	UsbMessageBuffer[OFFSET_BCLOCKSTATUS] = 0x01;
	return SLOT_NO_ERROR;
}

uint8_t PC_to_RDR_GetSlotStatus(void)
{
	uint8_t ErrorCode;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if (make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_ABRFU_3B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_3B+1] != 0
		|| UsbMessageBuffer[OFFSET_ABRFU_3B+2] != 0)
		return SLOTERR_BAD_ABRFU_3B;

//	ErrorCode = GetHwError(&HwErrorCode);
	return ErrorCode;
}

uint8_t PC_to_RDR_XfrBlock(void)
{
	uint8_t ErrorCode;
	uint32_t BlockSize, ExpectedAnswerSize;
	uint32_t UsbMessageLength;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	BlockSize = make32(&UsbMessageBuffer[OFFSET_DWLENGTH]);
	UsbMessageLength = USB_MESSAGE_HEADER_SIZE + BlockSize;

	if(UsbMessageLength > USB_MESSAGE_BUFFER_MAX_LENGTH)
		return SLOTERR_BAD_LENTGH;

	if( g_CCID_sDevice.bAbortRequestFlag )
		return SLOTERR_CMD_ABORTED;

//	ErrorCode = GetHwError(&HwErrorCode);
//	if(ErrorCode != SLOT_NO_ERROR)
//		return ErrorCode;

	ExpectedAnswerSize = make16(&UsbMessageBuffer[OFFSET_WLEVELPARAMETER]);

//	ErrorCode = XfrBlock(&UsbMessageBuffer[OFFSET_ABDATA], &BlockSize, ExpectedAnswerSize);

	/*===== for test =====*/
	ErrorCode = SLOT_NO_ERROR;
	/*===== end for test =====*/

	if( g_CCID_sDevice.bAbortRequestFlag )
		return SLOTERR_CMD_ABORTED;

	if(ErrorCode == SLOT_NO_ERROR)
	{
		UsbMessageBuffer[OFFSET_DWLENGTH] = (uint8_t) BlockSize;
		UsbMessageBuffer[OFFSET_DWLENGTH+1] = (uint8_t) (BlockSize>>8);
		UsbMessageBuffer[OFFSET_DWLENGTH+2] = (uint8_t) (BlockSize>>16);
		UsbMessageBuffer[OFFSET_DWLENGTH+3] = (uint8_t) (BlockSize>>24);
	}

	return ErrorCode;
}

uint8_t PC_to_RDR_GetParameters(void)
{
	uint8_t ErrorCode;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if (make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_ABRFU_3B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_3B+1] != 0
		|| UsbMessageBuffer[OFFSET_ABRFU_3B+2] != 0)
		return SLOTERR_BAD_ABRFU_3B;

//	ErrorCode = GetHwError(&HwErrorCode);
	return ErrorCode;
}

uint8_t PC_to_RDR_ResetParameters(void)
{
	uint8_t ErrorCode;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if (make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_ABRFU_3B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_3B+1] != 0
		|| UsbMessageBuffer[OFFSET_ABRFU_3B+2] != 0)
		return SLOTERR_BAD_ABRFU_3B;

	UsbMessageBuffer[OFFSET_ABDATA] = DEFAULT_FIDI;
	UsbMessageBuffer[OFFSET_ABDATA+1] = DEFAULT_T01CONVCHECKSUM;
	UsbMessageBuffer[OFFSET_ABDATA+2] = DEFAULT_GUARDTIME;
	UsbMessageBuffer[OFFSET_ABDATA+3] = DEFAULT_WAITINGINTEGER;
	UsbMessageBuffer[OFFSET_ABDATA+4] = DEFAULT_CLOCKSTOP;
	UsbMessageBuffer[OFFSET_ABDATA+5] = 0x00;
	UsbMessageBuffer[OFFSET_ABDATA+6] = 0x00;

//	ErrorCode = SetParameters(&UsbMessageBuffer[OFFSET_ABDATA], 0x00);
	if(ErrorCode != SLOT_NO_ERROR)
		return ErrorCode;

//	ErrorCode = GetHwError(&HwErrorCode);
	return ErrorCode;
}

uint8_t PC_to_RDR_SetParameters(void)
{
	uint8_t ErrorCode;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if ((make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0x05) &&
		(make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0x07))
		return SLOTERR_BAD_LENTGH;
	if( ((UsbMessageBuffer[OFFSET_DWLENGTH] == 0x05)
			&& (UsbMessageBuffer[OFFSET_BPROTOCOLNUM_OUT] != 0x00))
		|| ((UsbMessageBuffer[OFFSET_DWLENGTH] == 0x07)
			&& (UsbMessageBuffer[OFFSET_BPROTOCOLNUM_OUT] != 0x01)) )
		return SLOTERR_BAD_PROTOCOLNUM;
	if(UsbMessageBuffer[OFFSET_ABRFU_2B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_2B+1] != 0)
		return SLOTERR_BAD_ABRFU_2B;

//	ErrorCode = SetParameters(&UsbMessageBuffer[OFFSET_ABDATA], UsbMessageBuffer[OFFSET_BPROTOCOLNUM_OUT]);

	/*===== for test =====*/
	ErrorCode = SLOT_NO_ERROR;
	/*===== end for test =====*/

	if(ErrorCode != SLOT_NO_ERROR)
		return ErrorCode;

//	ErrorCode = GetHwError(&HwErrorCode);
	return ErrorCode;
}

uint8_t PC_to_RDR_Escape(void)
{
	uint8_t ErrorCode;
	uint32_t BlockSize, UsbMessageLength;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	BlockSize = make32(&UsbMessageBuffer[OFFSET_DWLENGTH]);
	UsbMessageLength = USB_MESSAGE_HEADER_SIZE + BlockSize;
	if(UsbMessageLength > USB_MESSAGE_BUFFER_MAX_LENGTH)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_ABRFU_3B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_3B+1] != 0
		|| UsbMessageBuffer[OFFSET_ABRFU_3B+2] != 0)
		return SLOTERR_BAD_ABRFU_3B;

	if( g_CCID_sDevice.bAbortRequestFlag )
		return SLOTERR_CMD_ABORTED;

//	ErrorCode = Escape(&UsbMessageBuffer[OFFSET_ABDATA], &BlockSize);

	if( g_CCID_sDevice.bAbortRequestFlag )
		return SLOTERR_CMD_ABORTED;

	if(ErrorCode != SLOT_NO_ERROR)
		return ErrorCode;

	UsbMessageBuffer[OFFSET_DWLENGTH] = (uint8_t) BlockSize;
	UsbMessageBuffer[OFFSET_DWLENGTH+1] = (uint8_t) (BlockSize>>8);
	UsbMessageBuffer[OFFSET_DWLENGTH+2] = (uint8_t) (BlockSize>>16);
	UsbMessageBuffer[OFFSET_DWLENGTH+3] = (uint8_t) (BlockSize>>24);

//	ErrorCode = GetHwError(&HwErrorCode);
	return ErrorCode;
}

uint8_t PC_to_RDR_IccClock(void)
{
	uint8_t ErrorCode;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if (make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_BCLOCKCOMMAND] > 0x01)
		return SLOTERR_BAD_CLOCKCOMMAND;
	if(UsbMessageBuffer[OFFSET_ABRFU_2B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_2B+1] != 0)
		return SLOTERR_BAD_ABRFU_2B;

//	ErrorCode = SetClock(UsbMessageBuffer[OFFSET_BCLOCKCOMMAND]);
	if(ErrorCode != SLOT_NO_ERROR)
		return ErrorCode;

//	ErrorCode = GetHwError(&HwErrorCode);
	return ErrorCode;
}

uint8_t PC_to_RDR_Abort(void)
{
	uint8_t ErrorCode;

	if(UsbMessageBuffer[OFFSET_BSLOT] != 0)
		return SLOTERR_BAD_SLOT;
	if (make32(&UsbMessageBuffer[OFFSET_DWLENGTH]) != 0)
		return SLOTERR_BAD_LENTGH;
	if(UsbMessageBuffer[OFFSET_ABRFU_3B] != 0 || UsbMessageBuffer[OFFSET_ABRFU_3B+1] != 0
		|| UsbMessageBuffer[OFFSET_ABRFU_3B+2] != 0)
		return SLOTERR_BAD_ABRFU_3B;

	if( (!g_CCID_sDevice.bAbortRequestFlag) /*|| (g_bSeq != UsbMessageBuffer[OFFSET_BSEQ]) 
		|| (g_bSlot != UsbMessageBuffer[OFFSET_BSLOT])*/)
		return SLOTERR_CMD_ABORTED;

	g_CCID_sDevice.bAbortRequestFlag = 0;

	return SLOT_NO_ERROR;
}

//======================================================
// Bulk-In Message Functions
//======================================================
void RDR_to_PC_DataBlock(uint8_t ErrorCode)
{
	UsbMessageBuffer[OFFSET_BMESSAGETYPE] = RDR_TO_PC_DATABLOCK;
//	UsbMessageBuffer[OFFSET_BSTATUS] = GetSlotStatus();
	UsbMessageBuffer[OFFSET_BSTATUS] = 0x00;

	UsbMessageBuffer[OFFSET_BERROR] = 0x80;
	if(ErrorCode != SLOT_NO_ERROR)
	{
		UsbMessageBuffer[OFFSET_BSTATUS] += 0x40;
		UsbMessageBuffer[OFFSET_DWLENGTH] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+1] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+2] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+3] = 0x00;
		UsbMessageBuffer[OFFSET_BERROR] = ErrorCode;
	}
	UsbMessageBuffer[OFFSET_BCHAINPARAMETER] = 0x00;
	g_CCID_sDevice.isBulkInReady = 1;
}

void RDR_to_PC_SlotStatus(uint8_t ErrorCode)
{
	UsbMessageBuffer[OFFSET_BMESSAGETYPE] = RDR_TO_PC_SLOTSTATUS;
//	UsbMessageBuffer[OFFSET_BSTATUS] = GetSlotStatus();

	UsbMessageBuffer[OFFSET_BSTATUS] = 0x01;

	UsbMessageBuffer[OFFSET_BERROR] = 0x80;
	if(ErrorCode != SLOT_NO_ERROR)
	{
		UsbMessageBuffer[OFFSET_BSTATUS] += 0x40;
		UsbMessageBuffer[OFFSET_DWLENGTH] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+1] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+2] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+3] = 0x00;
		UsbMessageBuffer[OFFSET_BERROR] = ErrorCode;
	}
//	UsbMessageBuffer[OFFSET_BCLOCKSTATUS] = GetClockStatus();
	g_CCID_sDevice.isBulkInReady = 1;
}

void RDR_to_PC_Parameters(uint8_t ErrorCode)
{
	UsbMessageBuffer[OFFSET_BMESSAGETYPE] = RDR_TO_PC_PARAMETERS;
//	UsbMessageBuffer[OFFSET_BSTATUS] = GetSlotStatus();
	UsbMessageBuffer[OFFSET_DWLENGTH+1] = 0x00;
	UsbMessageBuffer[OFFSET_DWLENGTH+2] = 0x00;
	UsbMessageBuffer[OFFSET_DWLENGTH+3] = 0x00;
	UsbMessageBuffer[OFFSET_BERROR] = 0x80;
	if(ErrorCode != SLOT_NO_ERROR)
	{
		UsbMessageBuffer[OFFSET_BSTATUS] += 0x40;
		UsbMessageBuffer[OFFSET_DWLENGTH] = 0x00;
		UsbMessageBuffer[OFFSET_BERROR] = ErrorCode;
		g_CCID_sDevice.isBulkInReady = 1;
		return;
	}

//	UsbMessageBuffer[OFFSET_BPROTOCOLNUM_IN] = GetParameters(&UsbMessageBuffer[OFFSET_ABDATA]);

	if(UsbMessageBuffer[OFFSET_BPROTOCOLNUM_IN])
		UsbMessageBuffer[OFFSET_DWLENGTH] = 0x07;
	else
		UsbMessageBuffer[OFFSET_DWLENGTH] = 0x05;
	g_CCID_sDevice.isBulkInReady = 1;
}

void RDR_to_PC_Escape(uint8_t ErrorCode)
{
	UsbMessageBuffer[OFFSET_BMESSAGETYPE] = RDR_TO_PC_ESCAPE;
//	UsbMessageBuffer[OFFSET_BSTATUS] = GetSlotStatus();
	UsbMessageBuffer[OFFSET_BERROR] = 0x80;
	if(ErrorCode != SLOT_NO_ERROR)
	{
		UsbMessageBuffer[OFFSET_BSTATUS] += 0x40;
		UsbMessageBuffer[OFFSET_DWLENGTH] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+1] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+2] = 0x00;
		UsbMessageBuffer[OFFSET_DWLENGTH+3] = 0x00;
		UsbMessageBuffer[OFFSET_BERROR] = ErrorCode;
	}
	UsbMessageBuffer[OFFSET_BRFU] = 0x00;
	g_CCID_sDevice.isBulkInReady = 1;
}

//======================================================
// Bulk-In Message Functions
//======================================================
void RDR_to_PC_NotifySlotChange(void)
{
	UsbIntMessageBuffer[OFFSET_INT_BMESSAGETYPE] = RDR_TO_PC_NOTIFYSLOTCHANGE;

#if 1
	UsbIntMessageBuffer[OFFSET_INT_BMSLOTICCSTATE] = 0x03;
#else
	if( GetSlotStatus() == CRD_NOTPRESENT )
		UsbIntMessageBuffer[OFFSET_INT_BMSLOTICCSTATE] = 0x02;
	else
		UsbIntMessageBuffer[OFFSET_INT_BMSLOTICCSTATE] = 0x03;
#endif
}

void RDR_to_PC_HardwareError(void)
{
	uint8_t ErrorCode, HwErrorCode;

	UsbIntMessageBuffer[OFFSET_INT_BMESSAGETYPE] = RDR_TO_PC_HARDWAREERROR;
	UsbIntMessageBuffer[OFFSET_INT_BSLOT] = 0x00;
	UsbIntMessageBuffer[OFFSET_INT_BSEQ] = UsbMessageBuffer[OFFSET_BSEQ];
//	ErrorCode = GetHwError(&HwErrorCode);
	UsbIntMessageBuffer[OFFSET_INT_BHARDWAREERRORCODE] = HwErrorCode;
}


//======================================================
// Not Support Message
//======================================================
void CmdNotSupported(void)
{
	UsbMessageBuffer[OFFSET_DWLENGTH] = 0x00;
	UsbMessageBuffer[OFFSET_DWLENGTH+1] = 0x00;
	UsbMessageBuffer[OFFSET_DWLENGTH+2] = 0x00;
	UsbMessageBuffer[OFFSET_DWLENGTH+3] = 0x00;
	UsbMessageBuffer[OFFSET_BSTATUS] = 0x40 /*+ GetSlotStatus()*/;
	UsbMessageBuffer[OFFSET_BERROR] = 0x00;
	UsbMessageBuffer[OFFSET_BCHAINPARAMETER] = 0x00;
}



